package game;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.image.BufferStrategy;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;

public class Board extends Canvas implements Runnable, MouseListener {
	public static final int CELL_SIZE = 16;
	private static final long serialVersionUID = 1L;
	
	public int w, h;
	boolean[] lastActive;
	boolean[] active;
	int[] direction;
	
	public Board(int w, int h) {
		Dimension size = new Dimension(256, 256);
		setPreferredSize(size);
		setMaximumSize(size);
		setMinimumSize(size);
		setSize(size);
		
		this.w = w;
		this.h = h;
		
		active = new boolean[w * h];
		for (int i = 0; i < active.length; i++)
			active[i] = false;
		lastActive = new boolean[w * h];
		for (int i = 0; i < lastActive.length; i++)
			lastActive[i] = false;
		
		direction = new int[w * h];
		for (int i = 0; i < direction.length; i++)
			direction[i] = 0;
		
		this.addMouseListener(this);
	}
	
	public void start() {
		Thread thread = new Thread(this);
		thread.start();
	}
	
	public void tick() {
		for (int i = 0; i < active.length; i++) {
			boolean tmp = lastActive[i];
			lastActive[i] = active[i];
			active[i] = tmp;
		}
		
		for (int y = 0; y < h; y++) {
			for (int x = 0; x < w; x++) {
				int d = direction[x + y * w];
				if (d == 0) {
					active[x + y * w] = false;
					continue;
				}
				
				boolean[] inputs = {
						x > 0 && lastActive[(x - 1) + y * w],
						y > 0 && lastActive[x + (y - 1) * w],
						x < w - 1 && lastActive[(x + 1) + y * w],
						y < h - 1 && lastActive[x + (y + 1) * w]
				};
				inputs[d - 1] = false;
				int count = 0;
				for (int i = 0; i < 4; i++) {
					if (inputs[i])
						count++;
				}
				
				active[x + y * w] = count < 2;
			}
		}
	}
	
	public void render() {
		
		BufferStrategy bs = this.getBufferStrategy();
		if (bs == null) {
			this.createBufferStrategy(3);
			bs = this.getBufferStrategy();
		}
		Graphics g = bs.getDrawGraphics();
		g.setColor(Color.black);
		g.fillRect(0, 0, w, h);
		renderBoard(g);
		g.dispose();
		bs.show();
	}
	
	public void click(int x, int y) {
		if (x < 0 || y < 0 || x >= w || y >= h)
			return;
		
		direction[x + y * w] = (direction[x + y * w] + 1) % 5;
	}
	
	private void renderBoard(Graphics g) {
		for (int y = 0; y < h; y++) {
			for (int x = 0; x < w; x++) {
				g.drawImage(NandGame.sheet.image, x * 16, y * 16, x * 16 + 16, y * 16 + 16, direction[x + y * w] * 16, active[x + y * w] ? 16 : 0, direction[x + y * w] * 16 + 16, (active[x + y * w] ? 16 : 0) + 16, null);
			}
		}
	}
	
	@Override
	public void run() {
		tick();
		render();
		
		ScheduledExecutorService executor = Executors.newSingleThreadScheduledExecutor();
		ScheduledFuture<?> oldFuture = executor.schedule(this, 10, TimeUnit.MILLISECONDS);
		try {
			oldFuture.get();
		}
		catch (InterruptedException e) {
			e.printStackTrace();
		}
		catch (ExecutionException e) {
			e.printStackTrace();
		}
	}
	
	@Override
	public void mouseClicked(MouseEvent e) {
		int xCell = e.getX() / Board.CELL_SIZE;
		int yCell = e.getY() / Board.CELL_SIZE;
		click(xCell, yCell);
		render();
	}
	
	@Override
	public void mouseEntered(MouseEvent arg0) {
	}
	
	@Override
	public void mouseExited(MouseEvent arg0) {
	}
	
	@Override
	public void mousePressed(MouseEvent arg0) {
	}
	
	@Override
	public void mouseReleased(MouseEvent arg0) {
	}
	
}
